# This file is automatically generated by the installer.
# Do not edit by hands.

module Alexandria
    class Preferences
        DEFAULT_VALUES = {"position"=>[0, 0],"size"=>[640, 480],"maximized"=>false,"sidepane_position"=>140,"view_as"=>0,"arrange_icons_mode"=>0,"reverse_icons"=>false,"cuecat_device"=>"/dev/scanners/cuecat","toolbar_visible"=>true,"sidepane_visible"=>true,"statusbar_visible"=>true,"col_authors_visible"=>true,"col_edition_visible"=>true,"col_isbn_visible"=>true,"col_publisher_visible"=>true,"col_rating_visible"=>true,"providers_priority"=>["Amazon", "BN", "MCU", "Proxis", "Amadeus", "IBS_it"]}
    end
end
